@@ -2,13 +2,13 @@
 // audioio-mp3.cpp: Interface for mp3 decoders and encoders that support 
 //                  input/output using standard streams. Defaults to
 //                  mpg123 and lame.
-// Copyright (C) 1999-2005 Kai Vehmanen
+// Copyright (C) 1999-2006 Kai Vehmanen
 // Note! Routines for parsing mp3 header information were taken from XMMS
 //       1.2.5's mpg123 plugin. Improvements to parsing logic were
 //       contributed by Julian Dobson.
 //
 // Attributes:
-//     eca-style-version: 3
+//     eca-style-version: 3 (see Ecasound Programmer's Guide)
 //
 // References:
 //     http://www.mp3-tech.org/programmer/frame_header.html
@@ -336,6 +336,7 @@ MP3FILE::MP3FILE(const std::string& name)
 
 MP3FILE::~MP3FILE(void)
 {
+  clean_child(true);
   if (is_open() == true) {
     close();
   }
@@ -363,8 +364,9 @@ void MP3FILE::open(void) throw(AUDIO_IO::SETUP_ERROR &)
 void MP3FILE::close(void)
 {
   if (pid_of_child() > 0) {
-      ECA_LOG_MSG(ECA_LOGGER::user_objects, "Cleaning child process." + kvu_numtostr(pid_of_child()) + ".");
-      clean_child();
+      ECA_LOG_MSG(ECA_LOGGER::user_objects, "Cleaning child process pid=" + kvu_numtostr(pid_of_child()) + ".");
+      /* note: mp3 input/output can handle SIGTERM */
+      clean_child(true);
       triggered_rep = false;
   }
 
@@ -429,7 +431,7 @@ void MP3FILE::seek_position(void)
   if (triggered_rep == true &&
       last_position_rep != position_in_samples()) {
     if (is_open() == true) {
-      ECA_LOG_MSG(ECA_LOGGER::user_objects, "Cleaning child process." + kvu_numtostr(pid_of_child()) + ".");
+      ECA_LOG_MSG(ECA_LOGGER::user_objects, "Cleaning child process pid=" + kvu_numtostr(pid_of_child()) + ".");
       clean_child();
       triggered_rep = false;
     }
