@@ -4,7 +4,7 @@
 // Copyright (C) 2005 Stuart Allie
 //
 // Attributes:
-//     eca-style-version: 3
+//     eca-style-version: 3 (see Ecasound Programmer's Guide)
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -185,7 +185,7 @@ ECA_ENGINE::ECA_ENGINE(ECA_CHAINSETUP* csetup)
   DBC_REQUIRE(csetup->is_enabled() == true);
   // --
 
-  ECA_LOG_MSG(ECA_LOGGER::system_objects, "Engine/Initializing");
+  ECA_LOG_MSG(ECA_LOGGER::system_objects, "ECA_ENGINE constructor");
 
   csetup_repp->toggle_locked_state(true);
 
@@ -209,7 +209,7 @@ ECA_ENGINE::ECA_ENGINE(ECA_CHAINSETUP* csetup)
  */
 ECA_ENGINE::~ECA_ENGINE(void)
 {
-  ECA_LOG_MSG(ECA_LOGGER::system_objects, "ECA_ENGINE destructor!");
+  ECA_LOG_MSG(ECA_LOGGER::system_objects, "ECA_ENGINE destructor");
 
   if (csetup_repp != 0) {
     command(ECA_ENGINE::ep_exit, 0.0f);
@@ -258,13 +258,15 @@ ECA_ENGINE::~ECA_ENGINE(void)
  * @post status() == ECA_ENGINE::engine_status_notready
  * @post is_valid() != true
  */
-void ECA_ENGINE::exec(bool batch_mode)
+int ECA_ENGINE::exec(bool batch_mode)
 {
   // --
   DBC_REQUIRE(csetup_repp != 0);
   DBC_REQUIRE(csetup_repp->is_enabled() == true);
   // --
 
+  int result = 0;
+
   csetup_repp->toggle_locked_state(true);
 
   batchmode_enabled_rep = batch_mode;
@@ -284,15 +286,20 @@ void ECA_ENGINE::exec(bool batch_mode)
 
   signal_exit();
 
-  if (outputs_finished_rep > 0) 
+  if (outputs_finished_rep > 0) {
     ECA_LOG_MSG(ECA_LOGGER::info, "WARNING: An output object has raised an error! "
 		"Possible causes: Out of disk space, permission denied, unable to launch external "
-		"applications needed in procesing, etc.");
+		"applications needed in processing, etc.");
+  }
 
   DBC_CHECK(status() == ECA_ENGINE::engine_status_stopped ||
 	    status() == ECA_ENGINE::engine_status_finished ||
 	    status() == ECA_ENGINE::engine_status_error);
 
+  if (status() == ECA_ENGINE::engine_status_error) {
+    result = -1;
+  }
+
   cleanup();
 
   ECA_LOG_MSG(ECA_LOGGER::user_objects, 
@@ -302,6 +309,8 @@ void ECA_ENGINE::exec(bool batch_mode)
   // --
   DBC_ENSURE(status() == ECA_ENGINE::engine_status_notready);
   // --
+
+  return result;
 }
 
 /**
@@ -1048,8 +1057,9 @@ void ECA_ENGINE::set_position_samples_live(SAMPLE_SPECS::sample_pos_t samples)
  */
 void ECA_ENGINE::change_position(double seconds)
 {
+  double curpos = csetup_repp->position_in_seconds_exact();
   conditional_stop();
-  csetup_repp->seek_position_in_seconds(csetup_repp->position_in_seconds() + seconds);
+  csetup_repp->seek_position_in_seconds(curpos + seconds);
   conditional_start();
 }
 
@@ -1192,20 +1202,20 @@ void ECA_ENGINE::interpret_queue(void)
 	// ---
       case ep_ctrl_select: { 
 	csetup_repp->active_ctrl_index_rep =  static_cast<size_t>(item.second);
-	if (csetup_repp->active_ctrl_index_rep - 1 < static_cast<int>((*chains_repp)[csetup_repp->active_ctrl_index_rep]->number_of_controllers()))
-	  (*chains_repp)[csetup_repp->active_ctrl_index_rep]->select_controller(csetup_repp->active_ctrl_index_rep);
+	if (csetup_repp->active_ctrl_index_rep - 1 < static_cast<int>((*chains_repp)[csetup_repp->active_chain_index_rep]->number_of_controllers()))
+	  (*chains_repp)[csetup_repp->active_chain_index_rep]->select_controller(csetup_repp->active_ctrl_index_rep);
 	else 
 	  csetup_repp->active_ctrl_index_rep = 0;
 	break;
       }
       case ep_ctrlp_select: { 
 	csetup_repp->active_ctrl_param_index_rep = static_cast<size_t>(item.second);
-	(*chains_repp)[csetup_repp->active_ctrl_index_rep]->select_controller_parameter(csetup_repp->active_ctrl_param_index_rep);
+	(*chains_repp)[csetup_repp->active_chain_index_rep]->select_controller_parameter(csetup_repp->active_ctrl_param_index_rep);
 	break;
       }
       case ep_ctrlp_value: { 
 	assert(chains_repp != 0);
-	(*chains_repp)[csetup_repp->active_ctrl_index_rep]->set_controller_parameter(item.second);
+	(*chains_repp)[csetup_repp->active_chain_index_rep]->set_controller_parameter(item.second);
 	break;
       }
 	// ---
@@ -1280,10 +1290,11 @@ void ECA_ENGINE::init_driver(void)
  */
 void ECA_ENGINE::init_prefill(void)
 {
+  int channels = (max_channels() > 0 ? max_channels() : 1);
   prefill_threshold_rep = 0;
 
   if (csetup_repp->max_buffers() == true) 
-    prefill_threshold_rep = ECA_ENGINE::prefill_threshold_constant / buffersize() / max_channels();
+    prefill_threshold_rep = ECA_ENGINE::prefill_threshold_constant / buffersize() / channels;
 
   if (prefill_threshold_rep < ECA_ENGINE::prefill_blocks_constant)
     prefill_threshold_rep = ECA_ENGINE::prefill_blocks_constant;
