@@ -730,18 +730,24 @@ string ECA_OBJECT_FACTORY::operator_parameters_to_eos(const OPERATOR* chainop)
 /**
  * Return a string compliant with Ecasound Option Syntax (EOS)
  * describing the object 'aiod'.
+ *
+ * @pre direction == "i" || direction == "o"
  */
-string ECA_OBJECT_FACTORY::audio_object_to_eos(const AUDIO_IO* aiod)
+string ECA_OBJECT_FACTORY::audio_object_to_eos(const AUDIO_IO* aiod, const std::string& direction)
 {
   MESSAGE_ITEM t;
-  string direction ("i");
-  if (aiod->io_mode() != AUDIO_IO::io_read) {
-    direction = "o";
-  }
   t << " -" << direction << ":";
   for(int n = 0; n < aiod->number_of_params(); n++) {
-    // FIXME: should quote/escape possible commas and whitespace
-    t << aiod->get_parameter(n + 1);
+    /* step: if parameter has commas, or whitespace, quote the whole parameter */
+    std::string param = aiod->get_parameter(n + 1);
+    if (find(param.begin(), param.end(), ',') != param.end() ||
+	find(param.begin(), param.end(), ' ') != param.end() ||
+	find(param.begin(), param.end(), '\t') != param.end()) {
+      param = std::string("\"") + param + std::string("\"");
+    }
+
+    /* step: add processed parameter to the EOS string */
+    t << param;
     if (n + 1 < aiod->number_of_params()) t << ",";
   }
 
