@@ -1,6 +1,9 @@
 // ------------------------------------------------------------------------
 // eca-session.cpp: Ecasound runtime setup and parameters.
-// Copyright (C) 1999-2003 Kai Vehmanen (kai.vehmanen@wakkanet.fi)
+// Copyright (C) 1999-2004 Kai Vehmanen
+//
+// Attributes:
+//     eca-style-version: 3
 //
 // This program is fre software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -42,6 +45,8 @@
 #include "audioio-mikmod.h"
 #include "audioio-timidity.h"
 #include "audioio-ogg.h"
+#include "audioio-flac.h"
+#include "audioio-aac.h"
 
 #include "osc-gen-file.h"
 
@@ -73,8 +78,6 @@ ECA_SESSION::ECA_SESSION(COMMAND_LINE& cline) throw(ECA_ERROR&)
 {
   int errors = 0;
 
-  ECA_LOG_MSG(ECA_LOGGER::subsystems, "Session created");
-
   set_defaults();
 
   cline.combine();
@@ -84,6 +87,10 @@ ECA_SESSION::ECA_SESSION(COMMAND_LINE& cline) throw(ECA_ERROR&)
   preprocess_options(&options);
   errors += interpret_general_options(options,&csoptions);
 
+  /* NOTE: must be printed after general options are parsed 
+  *        in case user has specified -q (quiet operation) */
+  ECA_LOG_MSG(ECA_LOGGER::subsystems, "Session created");
+
   if (errors > 0) {
     throw(ECA_ERROR("ECA-SESSION", "Errors parsing session-level options. Unable to create session."));
   }
@@ -107,7 +114,7 @@ ECA_SESSION::ECA_SESSION(COMMAND_LINE& cline) throw(ECA_ERROR&)
 	delete comline_setup;
       }
       else if (selected_chainsetup_repp->is_valid() != true) {
-	ECA_LOG_MSG(ECA_LOGGER::info, "(eca-session) Note! Unable to create a valid chainsetup from the command-line arguments.");
+	ECA_LOG_MSG(ECA_LOGGER::info, "Note! Unable to create a valid chainsetup from the command-line arguments.");
       }
     }
   }
@@ -123,13 +130,16 @@ void ECA_SESSION::set_defaults(void)
 
   ECA_RESOURCES ecaresources;
 
-  MP3FILE::set_mp3_input_cmd(ecaresources.resource("ext-cmd-mp3-input"));
-  MP3FILE::set_mp3_output_cmd(ecaresources.resource("ext-cmd-mp3-output"));
+  MP3FILE::set_input_cmd(ecaresources.resource("ext-cmd-mp3-input"));
+  MP3FILE::set_output_cmd(ecaresources.resource("ext-cmd-mp3-output"));
   MIKMOD_INTERFACE::set_mikmod_cmd(ecaresources.resource("ext-cmd-mikmod"));
   TIMIDITY_INTERFACE::set_timidity_cmd(ecaresources.resource("ext-cmd-timidity"));
-  OGG_VORBIS_INTERFACE::set_ogg_input_cmd(ecaresources.resource("ext-cmd-ogg-input"));
-  OGG_VORBIS_INTERFACE::set_ogg_output_cmd(ecaresources.resource("ext-cmd-ogg-output"));
-
+  OGG_VORBIS_INTERFACE::set_input_cmd(ecaresources.resource("ext-cmd-ogg-input"));
+  OGG_VORBIS_INTERFACE::set_output_cmd(ecaresources.resource("ext-cmd-ogg-output"));
+  FLAC_FORKED_INTERFACE::set_input_cmd(ecaresources.resource("ext-cmd-flac-input"));
+  FLAC_FORKED_INTERFACE::set_output_cmd(ecaresources.resource("ext-cmd-flac-output"));
+  AAC_FORKED_INTERFACE::set_input_cmd(ecaresources.resource("ext-cmd-aac-input"));
+  AAC_FORKED_INTERFACE::set_output_cmd(ecaresources.resource("ext-cmd-aac-output"));
 }
 
 
@@ -192,7 +202,7 @@ void ECA_SESSION::add_chainsetup(ECA_CHAINSETUP* comline_setup)
   while(p != chainsetups_rep.end()) {
     if ((*p)->name() == comline_setup->name()) {
       ECA_LOG_MSG(ECA_LOGGER::info, 
-		  "(eca-session) Unable to add chainsetup, chainsetup with the same name already exists.");
+		  "Unable to add chainsetup, chainsetup with the same name already exists.");
       selected_chainsetup_repp = 0;
       break;
     }
@@ -254,7 +264,7 @@ void ECA_SESSION::select_chainsetup(const std::string& name)
   std::vector<ECA_CHAINSETUP*>::const_iterator p = chainsetups_rep.begin();
   while(p != chainsetups_rep.end()) {
     if ((*p)->name() == name) {
-      //  ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-session) Chainsetup \"" + name + "\" selected.");
+      //  ECA_LOG_MSG(ECA_LOGGER::system_objects, "Chainsetup \"" + name + "\" selected.");
       selected_chainsetup_repp = *p;
       break;
     }
@@ -307,7 +317,7 @@ void ECA_SESSION::load_chainsetup(const std::string& filename)
     string temp = new_setup->interpret_result_verbose();
     delete new_setup;
     selected_chainsetup_repp = 0;
-    ECA_LOG_MSG(ECA_LOGGER::info, "(eca-session) Error loading chainsetup: " + temp);
+    ECA_LOG_MSG(ECA_LOGGER::info, "Error loading chainsetup: " + temp);
   }
   else {
     add_chainsetup(new_setup);
@@ -378,7 +388,7 @@ std::vector<std::string> ECA_SESSION::chainsetup_names(void) const
     result.push_back((*p)->name());
     ++p;
   }
-  return(result);
+  return result;
 }
 
 void ECA_SESSION::create_chainsetup_options(COMMAND_LINE& cline,
@@ -398,17 +408,17 @@ void ECA_SESSION::create_chainsetup_options(COMMAND_LINE& cline,
 bool ECA_SESSION::is_session_option(const std::string& arg) const
 {
   if (arg.size() < 2 ||
-      arg[0] != '-') return(false);
+      arg[0] != '-') return false;
 
   switch(arg[1]) {
   case 'd':
   case 'q':
-    return(true);
+    return true;
 
   case 's': 
-    if (arg.size() > 2 && arg[2] == ':') return(true);
+    if (arg.size() > 2 && arg[2] == ':') return true;
   }
-  return(false);
+  return false;
 }
 
 /**
@@ -429,7 +439,7 @@ void ECA_SESSION::preprocess_options(std::vector<std::string>* opts)
       /* hack1: options ending with .ecs as "-s:file.ecs" */
       string::size_type pos = p->find(".ecs");
       if (pos + 4 == p->size()) {
-	ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) Note! Interpreting option " +
+	ECA_LOG_MSG(ECA_LOGGER::info, "Note! Interpreting option " +
 		    *p +
 		    " as -s:" +
 		    *p +
@@ -469,7 +479,7 @@ int ECA_SESSION::interpret_general_options(const std::vector<std::string>& inopt
     ++p;
   }
 
-  return(errors);
+  return errors;
 }
 
 /**
@@ -479,8 +489,8 @@ int ECA_SESSION::interpret_general_options(const std::vector<std::string>& inopt
  */
 int ECA_SESSION::interpret_general_option (const std::string& argu)
 {
-  if (argu.size() < 2) return(0);
-  if (argu[0] != '-') return(0);
+  if (argu.size() < 2) return 0;
+  if (argu[0] != '-') return 0;
 
   switch(argu[1]) {
   case 'd':
@@ -496,6 +506,7 @@ int ECA_SESSION::interpret_general_option (const std::string& argu)
 	ECA_LOGGER::instance().set_log_level(ECA_LOGGER::errors, 1);
 	ECA_LOGGER::instance().set_log_level(ECA_LOGGER::info, 1);
 	ECA_LOGGER::instance().set_log_level(ECA_LOGGER::subsystems, 1);
+	ECA_LOGGER::instance().set_log_level(ECA_LOGGER::module_names, 1);
 	ECA_LOGGER::instance().set_log_level(ECA_LOGGER::user_objects, 1);
 	ECA_LOGGER::instance().set_log_level(ECA_LOGGER::eiam_return_values, 1);
 
@@ -505,7 +516,6 @@ int ECA_SESSION::interpret_general_option (const std::string& argu)
 
 	  if (argu.size() > 3 && argu[3] == 'd') {
 	    /* argu == "-ddd" */
-	    ECA_LOGGER::instance().set_log_level(ECA_LOGGER::module_names, 1);
 	    ECA_LOGGER::instance().set_log_level(ECA_LOGGER::functions, 1);
 	    ECA_LOGGER::instance().set_log_level(ECA_LOGGER::continuous, 1);
 	  }
@@ -513,7 +523,7 @@ int ECA_SESSION::interpret_general_option (const std::string& argu)
       }
 
       MESSAGE_ITEM mtempd;
-      mtempd << "(eca-session) Set debug level to: " << ECA_LOGGER::instance().get_log_level_bitmask();
+      mtempd << "Set debug level to: " << ECA_LOGGER::instance().get_log_level_bitmask();
       ECA_LOG_MSG(ECA_LOGGER::info, mtempd.to_string());
       break;
     }
@@ -525,7 +535,7 @@ int ECA_SESSION::interpret_general_option (const std::string& argu)
   default: { }
   }
 
-  return(0);
+  return 0;
 }
 
 /**
@@ -537,18 +547,18 @@ int ECA_SESSION::interpret_chainsetup_option (const std::string& argu)
 {
   int errors = 0;
 
-  if (argu.size() == 0) return(errors);
+  if (argu.size() == 0) return errors;
   
   string tname = kvu_get_argument_number(1, argu);
  
-  if (argu.size() < 2) return(errors);
+  if (argu.size() < 2) return errors;
   switch(argu[1]) {
   case 's': {
     if (argu.size() > 2 && argu[2] == ':') {
       load_chainsetup(tname);
       if (selected_chainsetup_repp == 0 || 
-	  selected_chainsetup_repp->is_valid() != true) {
-	ECA_LOG_MSG(ECA_LOGGER::info, "(eca-session) Chainsetup loaded from '" + tname + "' is not valid!");
+	  selected_chainsetup_repp->is_valid_for_connection(true) != true) {
+	ECA_LOG_MSG(ECA_LOGGER::info, "Chainsetup loaded from '" + tname + "' is not valid!");
 	++errors;
       }
       if (errors == 0) connect_chainsetup();
@@ -558,5 +568,5 @@ int ECA_SESSION::interpret_chainsetup_option (const std::string& argu)
   default: { }
   }
   
-  return(errors);
+  return errors;
 }
